MODULE SPI1;
(* =========================================================================  
   SPI1 - Serial Peripheral Interface Driver
   
   Target: RISC5 Project Oberon
   
   (c) 2017 CFB Software   
   http://www.astrobe.com  

  Permission to use, copy, modify, and/or distribute this software and its
  accompanying documentation (the "Software") for any purpose with or
  without fee is hereby granted, provided that the above copyright notice
  and this permission notice appear in all copies.
  
  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHORS DISCLAIM ALL WARRANTIES
  WITH REGARD TO THE SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
  MERCHANTABILITY, FITNESS AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
  AUTHORS BE LIABLE FOR ANY CLAIM, SPECIAL, DIRECT, INDIRECT, OR
  CONSEQUENTIAL DAMAGES OR ANY DAMAGES OR LIABILITY WHATSOEVER, WHETHER IN
  AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  CONNECTION WITH THE DEALINGS IN OR USE OR PERFORMANCE OF THE SOFTWARE.
  ========================================================================= *)

IMPORT SYSTEM;

CONST 
  (* SPI1 registers *)
  SPI1 = 1;
  SPIData = -48; 
  SPICtrl = -44;


PROCEDURE Select*(selected: BOOLEAN);
BEGIN
  IF selected THEN 
    SYSTEM.PUT(SPICtrl, {SPI1})
  ELSE
    SYSTEM.PUT(SPICtrl, {})
  END 
END Select;


PROCEDURE SendByte*(data: BYTE);
BEGIN
  SYSTEM.PUT(SPIData, data);
  REPEAT UNTIL SYSTEM.BIT(SPICtrl, 0);
END SendByte;  


PROCEDURE ReceiveByte*(VAR data: BYTE);
BEGIN
  SYSTEM.PUT(SPIData, 0FFH);
  REPEAT UNTIL SYSTEM.BIT(SPICtrl, 0);
  SYSTEM.GET(SPIData, data); 
END ReceiveByte;  


PROCEDURE Send*(b: ARRAY OF BYTE);
VAR
  i: INTEGER; 
BEGIN 
  FOR i := 0 TO LEN(b) - 1 DO 
    SYSTEM.PUT(SPIData, b[i]);
    REPEAT UNTIL SYSTEM.BIT(SPICtrl, 0);
  END
END Send; 


PROCEDURE Receive*(VAR b: ARRAY OF BYTE);
VAR
  i: INTEGER; 
BEGIN
  FOR i := 0 TO LEN(b) - 1 DO 
    SYSTEM.PUT(SPIData, 0FFH);
    REPEAT UNTIL SYSTEM.BIT(SPICtrl, 0);
    SYSTEM.GET(SPIData, b[i]);
  END
END Receive;  

END SPI1.
